# -*- coding: utf-8 -*-
# Author: Carlos Vigo
# Contact: carviher1990@gmail.com

""" PDF parser for credit card statements.
It accepts statement from the following issuers:
 - Cembra (Cumulus MasterCard)
 - Swisscard Cashback (AMEX / VISA / MasterCard)
The data can be saved to a CSV file compatible with Wallet import feature.
"""

# Local imports
from . import __project__, pdf_importer

__all__ = [
    __project__.__author__,
    __project__.__copyright__,
    __project__.__short_version__,
    __project__.__version__,
    __project__.__project_name__,
    'pdf_importer',
]



""" Collection of importers.
"""

# Imports

# Third party
import camelot
from dateutil.parser import parse
from pandas import concat


def extract_cembra(filename):
    entries = []

    # noinspection PyUnresolvedReferences
    tables = camelot.read_pdf(filename, pages='2-end', flavor='stream', table_areas=['60,670,600,100'])
    df = tables[0].df
    new_header = df.iloc[0]
    df = df[1:]
    df.columns = new_header

    for _, row in df.iterrows():
        try:
            date = parse(row[1].strip(), dayfirst=True).date()
            date2 = parse(row[0].strip(), dayfirst=True).date()
            text = row[2]
            credit = row[3].replace('\'', '')
            debit = row[4].replace('\'', '')
            amount = -float(debit) if debit else float(credit)
            entries.append([date, amount, text])
        except ValueError:
            pass

    return entries


def extract_cashback(filename):
    entries = []

    # noinspection PyUnresolvedReferences
    table1 = camelot.read_pdf(
        filename,
        pages='1',
        flavor='stream',
        table_areas=['50,320,560,50'],
        columns=['120,530']
    )
    table2 = camelot.read_pdf(
        filename,
        pages='2-end',
        flavor='stream',
        table_areas=['50,800,560,50'],
        columns=['120,530']
    )

    df = concat([table1[0].df, table2[0].df])

    for index, row in df.iterrows():
        try:
            date = parse(row[0].strip(), dayfirst=True).date()
            text = row[1].replace("\n", " ")
            amount = -float(row[2])

            entries.append([date, amount, text])
        except ValueError:
            pass

    return entries



__author__ = 'Carlos Vigo <carviher1990@gmailcom>'
__email__ = '<carlosv@phys.ethz.ch>'
__short_author__ = 'Carlos Vigo'
__copyright__ = '2021, Carlos Vigo'
__package_name__ = 'pdf-importer'
__module_name__ = 'pdf_importer.py'
__project_name__ = 'Statement PDF Importer'
__url__ = 'https://github.com/c-vigo/StatementPDFImporter'
__documentation__ = 'https://github.com/c-vigo/StatementPDFImporter'
__version__ = '0.4'
__short_version__ = '0.4'
__description__ = 'A PDF importer to generate CSV files from bank statements'




""" PDF parser for credit card statements.
It accepts statement from the following issuers:
 - Cembra (Cumulus MasterCard)
 - Swisscard Cashback (AMEX / VISA / MasterCard)
The data can be saved to a CSV file compatible with Wallet import feature.
"""

# Imports
import csv
from argparse import ArgumentParser

# Local packages
from .__project__ import (
    __documentation__ as docs_url,
    __module_name__ as module,
    __description__ as prog_desc,
)
from .Importers import extract_cembra, extract_cashback


def pdf_importer():
    """The main routine. It parses the input argument and acts accordingly."""

    # The argument parser
    ap = ArgumentParser(
        prog=module,
        description=prog_desc,
        add_help=True,
        epilog='Check out the package documentation for more information:\n{}'.format(docs_url)
    )

    # Positional arguments:
    # 1. File name
    ap.add_argument(
        'filename',
        help='PDF file to parse',
        type=str,
    )

    # 2. Statement type
    ap.add_argument(
        'type',
        help='statement type',
        type=str,
        choices=['cembra', 'cashback']
    )

    # 3. Output file
    ap.add_argument(
        '--o',
        '-output',
        dest='output',
        help='output CSV file',
        type=str,
        default=None
    )

    # Parse the arguments
    args = ap.parse_args()

    # Extract data from PDF
    if args.type == 'cembra':
        print('Processing file {} as a Cembra PDF statement.'.format(args.filename))
        entries = extract_cembra(args.filename)
    elif args.type == 'cashback':
        entries = extract_cashback(args.filename)
    else:
        raise RuntimeError('Invalid statement type {}'.format(args.type))

    # Print to console
    total_value = 0.
    for entry in entries:
        print('{}  {:+7.2f}   {}'.format(entry[0], entry[1], entry[2]))
        total_value += entry[1]
    print('\nTotal: {:+7.2f}'.format(total_value))

    # Save to CSV file
    if args.output is not None:
        print('Saving data to {}.'.format(args.output))
        with open(args.output, mode='w') as f:
            writer = csv.writer(
                f,
                delimiter=';',
                quotechar='"',
                quoting=csv.QUOTE_MINIMAL
            )
            for entry in entries:
                writer.writerow(entry)
